@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2017 Free
+   Copyright (C) 1999-2001, 2005, 2007, 2009-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -406,7 +406,7 @@ message_finalize (mu_message_t msg, int warn)
 	  return 1;
 	}
       sprintf (warn, "%s %s", pwd->pw_name, SENDER_WARNING);
-      mu_header_set_value (header, "X-Authentication-Warning", warn, 0);
+      mu_header_append (header, "X-Authentication-Warning", warn);
       free (warn);
     }
   
