@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2007-2012, 2014-2018 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,35 +13,33 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
 int
-imap4d_idle (struct imap4d_command *command, imap4d_tokbuf_t tok)
+imap4d_idle (struct imap4d_session *session,
+             struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   time_t start;
   char *token_str = NULL;
   size_t token_size = 0, token_len;
   
   if (imap4d_tokbuf_argc (tok) != 2)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
 
-  if (util_wait_input (0) == -1)
-    return util_finish (command, RESP_NO, "Cannot idle");
+  if (io_wait_input (0) == -1)
+    return io_completion_response (command, RESP_NO, "Cannot idle");
 
-  util_send ("+ idling\r\n");
-  util_flush_output ();
+  io_sendf ("+ idling\n");
+  io_flush ();
 
   start = time (NULL);
   while (1)
     {
-      if (util_wait_input (5))
+      if (io_wait_input (5))
 	{
-          imap4d_getline (&token_str, &token_size, &token_len); 	  
-	  token_len = util_trim_nl (token_str, token_len);
+          io_getline (&token_str, &token_size, &token_len); 	  
 	  if (token_len == 4 && mu_c_strcasecmp (token_str, "done") == 0)
 	    break;
 	}
@@ -48,9 +47,9 @@ imap4d_idle (struct imap4d_command *command, imap4d_tokbuf_t tok)
 	imap4d_bye (ERR_TIMEOUT);
 
       imap4d_sync ();
-      util_flush_output ();
+      io_flush ();
     }
   free (token_str);
-  return util_finish (command, RESP_OK, "terminated");
+  return io_completion_response (command, RESP_OK, "terminated");
 }
 
