@@ -1,7 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009, 2010 Free Software Foundation,
-   Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006-2007, 2009-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -14,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MUCPP_MAILER_H
 #define _MUCPP_MAILER_H
@@ -25,7 +22,6 @@
 #include <string>
 #include <errno.h>
 #include <mailutils/mailer.h>
-#include <mailutils/cpp/debug.h>
 #include <mailutils/cpp/message.h>
 #include <mailutils/cpp/address.h>
 
@@ -48,7 +44,6 @@ class Mailer
   void send_message (const Message& msg, const Address& from,
 		     const Address& to);
 
-  Debug& get_debug ();
 };
 
 }
