@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2008, 2010-2012, 2014-2018 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_PROGMAILER_H
 #define _MAILUTILS_PROGMAILER_H
@@ -28,8 +28,6 @@ extern "C" {
 int mu_progmailer_create (mu_progmailer_t *pm);
 int mu_progmailer_set_command (mu_progmailer_t pm, const char *command);
 int mu_progmailer_sget_command (mu_progmailer_t pm, const char **command);
-  /* FIXME: missing _aget_ and _get_ */
-int mu_progmailer_set_debug (mu_progmailer_t pm, mu_debug_t debug);  
 int mu_progmailer_open (mu_progmailer_t pm, char **argv);
 int mu_progmailer_send (mu_progmailer_t pm, mu_message_t msg);
 int mu_progmailer_close (mu_progmailer_t pm);
