@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2009-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,19 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 #include <mailutils/mailutils.h>
-#include <xalloc.h>
 #include <fnmatch.h>
-#define obstack_chunk_alloc malloc
-#define obstack_chunk_free free
-#include <obstack.h>  
 #include <sys/wait.h>
 #include <ctype.h>
 
@@ -55,56 +50,62 @@ struct mime_context
   char *temp_file;
   int unlink_temp_file;
 
-  char *no_ask_str;
   mu_list_t no_ask_types;
-  int debug_level;
+  int dh;
   int flags;
 };
 
-#define DEBUG(c,l,f) if ((c)->debug_level > (l)) printf f
-
 static int
 mime_context_fill (struct mime_context *ctx, const char *file,
 		   mu_stream_t input, mu_header_t hdr, const char *no_ask,
-		   int interactive, int dry_run, int debug_level)
+		   int interactive, int dry_run, mu_debug_handle_t dh)
 {
-  char *p, *sp;
- 
+  struct mu_wordsplit ws;
+  size_t i;
+  
   memset (ctx, 0, sizeof *ctx);
   ctx->input = input;
   ctx->hdr = hdr;
   if (mu_header_aget_value (hdr, MU_HEADER_CONTENT_TYPE,
-			 &ctx->content_type_buffer))
+			    &ctx->content_type_buffer))
     return 1;
-  ctx->content_type = strtok_r (ctx->content_type_buffer, ";", &sp);
-  ctx->temp_file = file ? strdup (file) : NULL; 
+  ws.ws_delim = ";";
+  if (mu_wordsplit (ctx->content_type_buffer, &ws,
+		    MU_WRDSF_DELIM|MU_WRDSF_SQUEEZE_DELIMS|MU_WRDSF_WS|
+		    MU_WRDSF_NOVAR|MU_WRDSF_NOCMD))
+    {
+      mu_error (_("cannot split line `%s': %s"),
+		ctx->content_type_buffer,
+		mu_wordsplit_strerror (&ws));
+      return 1;
+    }
+
+  ctx->content_type = ws.ws_wordv[0];
+  ws.ws_wordv[0] = NULL;
+  ctx->temp_file = file ? mu_strdup (file) : NULL; 
   ctx->unlink_temp_file = 0;
 
   if (interactive)
     ctx->flags |= FLAGS_INTERACTIVE;
   if (dry_run)
     ctx->flags |= FLAGS_DRY_RUN;
-  ctx->debug_level = debug_level;
+  ctx->dh = dh;
   
   mu_list_create (&ctx->values);
-  while ((p = strtok_r (NULL, ";", &sp)))
+
+  for (i = 1; i < ws.ws_wordc; i++)
     {
-      while (*p && isspace (*p))
-	p++;
-      mu_list_append (ctx->values, p);
+      mu_list_append (ctx->values, ws.ws_wordv[i]);
+      ws.ws_wordv[i] = NULL;
     }
+  mu_wordsplit_free (&ws);
   
   if (no_ask)
     {
-      ctx->no_ask_str = xstrdup (no_ask);
       mu_list_create (&ctx->no_ask_types);
-      for (p = strtok_r (ctx->no_ask_str, ",", &sp); p;
-	   p = strtok_r (NULL, ",", &sp))
-	{
-	  while (*p && isspace (*p))
-	    p++;
-	  mu_list_append (ctx->no_ask_types, p);
-	}
+      mu_list_set_destroy_item (ctx->no_ask_types, mu_list_free_item);
+      if (mu_string_split (no_ask, ",", ctx->no_ask_types))
+	return 1;
     }
   return 0;
 }
@@ -117,7 +118,6 @@ mime_context_release (struct mime_context *ctx)
     unlink (ctx->temp_file);
   free (ctx->temp_file);
   mu_list_destroy (&ctx->values);
-  free (ctx->no_ask_str);
   mu_list_destroy (&ctx->no_ask_types);
 }
 
@@ -199,6 +199,7 @@ mime_context_get_content_type_value (struct mime_context *ctx,
   return rc;
 }
 
+/* FIXME: Rewrite via mu_stream_copy */
 static void
 mime_context_write_input (struct mime_context *ctx, int fd)
 {
@@ -208,8 +209,13 @@ mime_context_write_input (struct mime_context *ctx, int fd)
   int status;
   
   mime_context_get_input (ctx, &input);
-  mu_stream_seek (input, 0, SEEK_SET);
-  while ((status = mu_stream_sequential_read (input, buf, sizeof buf, &n)) == 0
+  status = mu_stream_seek (input, 0, SEEK_SET, NULL);
+  if (status && status != ENOSYS)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_stream_seek", NULL, status);
+      abort (); /* FIXME */
+    }
+  while ((status = mu_stream_read (input, buf, sizeof buf, &n)) == 0
 	 && n)
     write (fd, buf, n);
 }
@@ -219,8 +225,8 @@ mime_context_get_temp_file (struct mime_context *ctx, char **ptr)
 {
   if (!ctx->temp_file)
     {
-      int fd = mu_tempfile (NULL, &ctx->temp_file);
-      if (fd == -1)
+      int fd;
+      if (mu_tempfile (NULL, 0, &fd, &ctx->temp_file))
 	return -1;
       mime_context_write_input (ctx, fd);
       close (fd);
@@ -231,7 +237,7 @@ mime_context_get_temp_file (struct mime_context *ctx, char **ptr)
 }
 
 
-static struct obstack expand_stack;
+static mu_opool_t expand_pool;
 
 static int
 expand_string (struct mime_context *ct, char **pstr)
@@ -248,14 +254,14 @@ expand_string (struct mime_context *ct, char **pstr)
 	      {
 	      case 's':
 		mime_context_get_temp_file (ct, &s);
-		obstack_grow (&expand_stack, s, strlen (s));
+		mu_opool_appendz (expand_pool, s);
 		rc = 1;
 		p += 2;
 		break;
 		
 	      case 't':
 		mime_context_get_content_type (ct, &s);
-		obstack_grow (&expand_stack, s, strlen (s));
+		mu_opool_appendz (expand_pool, s);
 		p += 2;
 		break;
 		
@@ -271,7 +277,7 @@ expand_string (struct mime_context *ct, char **pstr)
 		  if (mime_context_get_content_type_value (ct,
 							   q, p-q,
 							   &s, &n) == 0)
-		    obstack_grow (&expand_stack, s, n);
+		    mu_opool_append (expand_pool, s, n);
 		  if (*p)
 		    p++;
 		  break;
@@ -283,19 +289,19 @@ expand_string (struct mime_context *ct, char **pstr)
 		break;
 		
 	      default:
-		obstack_1grow (&expand_stack, p[0]);
+		mu_opool_append_char (expand_pool, p[0]);
 	      }
 	    break;
 
 	case '\\':
 	  if (p[1])
 	    {
-	      obstack_1grow (&expand_stack, p[1]);
+	      mu_opool_append_char (expand_pool, p[1]);
 	      p += 2;
 	    }
 	  else
 	    {
-	      obstack_1grow (&expand_stack, p[0]);
+	      mu_opool_append_char (expand_pool, p[0]);
 	      p++;
 	    }
 	  break;
@@ -303,23 +309,23 @@ expand_string (struct mime_context *ct, char **pstr)
 	case '"':
 	  if (p[1] == p[0])
 	    {
-	      obstack_1grow (&expand_stack, '%');
+	      mu_opool_append_char (expand_pool, '%');
 	      p++;
 	    }
 	  else
 	    {
-	      obstack_1grow (&expand_stack, p[0]);
+	      mu_opool_append_char (expand_pool, p[0]);
 	      p++;
 	    }
 	  break;
 
 	default:
-	  obstack_1grow (&expand_stack, p[0]);
+	  mu_opool_append_char (expand_pool, p[0]);
 	  p++;
 	}
     }
-  obstack_1grow (&expand_stack, 0);
-  *pstr = obstack_finish (&expand_stack);
+  mu_opool_append_char (expand_pool, 0);
+  *pstr = mu_opool_finish (expand_pool, NULL);
   return rc;
 }
 
@@ -331,7 +337,7 @@ confirm_action (struct mime_context *ctx, const char *str)
   char *type;
 
   mime_context_get_content_type (ctx, &type);
-  if (dry_run_p (ctx) || !interactive_p (ctx) || mime_context_do_not_ask (ctx))
+  if (!interactive_p (ctx) || mime_context_do_not_ask (ctx))
     return 1;
   
   printf (_("Run `%s'?"), str);
@@ -375,7 +381,7 @@ dump_mailcap_entry (mu_mailcap_entry_t entry)
 		    mu_strerror (status));
 	  break;
 	}
-      printf ("fields[%d]: %s\n", i, buffer);
+      printf ("fields[%u]: %s\n", (unsigned) i, buffer);
     }
   printf ("\n");
 }
@@ -414,21 +420,28 @@ create_filter (char *cmd, int outfd, int *infd)
   if (pid == 0)
     {
       /* Child process */
-      int argc;
+      struct mu_wordsplit ws;
       char **argv;
-
+      
       if (need_shell_p (cmd))
 	{
-	  argc = 3;
-	  argv = xmalloc ((argc + 1) * sizeof *argv);
+	  char *x_argv[4];
+	  argv = x_argv;
 	  argv[0] = getenv ("SHELL");
 	  argv[1] = "-c";
 	  argv[2] = cmd;
 	  argv[3] = NULL;
 	}
       else
-	mu_argcv_get (cmd, "", NULL, &argc, &argv);
-      
+	{
+	  if (mu_wordsplit (cmd, &ws, MU_WRDSF_DEFFLAGS))
+	    {
+	      mu_error (_("%s failed: %s"), "mu_wordsplit",
+			mu_wordsplit_strerror (&ws));
+	      _exit (127);
+	    }
+	  argv = ws.ws_wordv;
+	}      
       /* Create input channel: */
       if (infd)
 	{
@@ -440,9 +453,9 @@ create_filter (char *cmd, int outfd, int *infd)
       /* Create output channel */
       if (outfd != -1 && outfd != 1)
 	dup2 (outfd, 1);
-
+      
       execvp (argv[0], argv);
-      mu_error (_("Cannot execute `%s': %s"), cmd, mu_strerror (errno));
+      mu_error (_("cannot execute `%s': %s"), cmd, mu_strerror (errno));
       _exit (127);
     }
 
@@ -491,20 +504,24 @@ run_test (mu_mailcap_entry_t entry, struct mime_context *ctx)
   
   if (mu_mailcap_entry_get_test (entry, NULL, 0, &size) == 0)
     {
-      int argc;
-      char **argv;
+      struct mu_wordsplit ws;
       char *str;
 
-      obstack_blank (&expand_stack, size + 1);
-      str = obstack_finish (&expand_stack);
+      mu_opool_alloc (expand_pool, size + 1);
+      str = mu_opool_finish (expand_pool, NULL);
       mu_mailcap_entry_get_test (entry, str, size + 1, NULL);
 
       expand_string (ctx, &str);
-      mu_argcv_get (str, "", NULL, &argc, &argv);
+      if (mu_wordsplit (str, &ws, MU_WRDSF_DEFFLAGS))
+	{
+	  mu_error (_("%s failed: %s"), "mu_wordsplit",
+		    mu_wordsplit_strerror (&ws));
+	  return 1;
+	}
       
-      if (mu_spawnvp (argv[0], argv, &status))
+      if (mu_spawnvp (ws.ws_wordv[0], ws.ws_wordv, &status))
 	status = 1;
-      mu_argcv_free (argc, argv);
+      mu_wordsplit_free (&ws);
     }
   return status;
 }
@@ -519,9 +536,9 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
   int fd;                
   int *pfd = NULL;      
   int outfd = -1;       
-  pid_t pid, pager_pid;
+  pid_t pid;
   
-  if (ctx->debug_level > 1)
+  if (mu_debug_level_p (ctx->dh, MU_DEBUG_TRACE3))
     dump_mailcap_entry (entry);
 
   if (run_test (entry, ctx))
@@ -532,8 +549,8 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
       if (mu_mailcap_entry_get_viewcommand (entry, NULL, 0, &size))
 	return 1;
       size++;
-      obstack_blank (&expand_stack, size);
-      view_command = obstack_finish (&expand_stack);
+      mu_opool_alloc (expand_pool, size);
+      view_command = mu_opool_finish (expand_pool, NULL);
       mu_mailcap_entry_get_viewcommand (entry, view_command, size, NULL);
     }
   else
@@ -541,8 +558,8 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
       if (mu_mailcap_entry_get_value (entry, "print", NULL, 0, &size))
 	return 1;
       size++;
-      obstack_blank (&expand_stack, size);
-      view_command = obstack_finish (&expand_stack);
+      mu_opool_alloc (expand_pool, size);
+      view_command = mu_opool_finish (expand_pool, NULL);
       mu_mailcap_entry_get_value (entry, "print", view_command, size, NULL);
     }
 
@@ -552,15 +569,17 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
     pfd = NULL;
   else
     pfd = &fd;
-  DEBUG (ctx, 0, (_("Executing %s...\n"), view_command));
+  mu_debug (ctx->dh, MU_DEBUG_TRACE0, (_("executing %s...\n"), view_command));
 
   if (!confirm_action (ctx, view_command))
     return 1;
-    
+  if (dry_run_p (ctx))
+    return 0;
+  
   flag = 0;
   if (interactive_p (ctx)
       && mu_mailcap_entry_copiousoutput (entry, &flag) == 0 && flag)
-    pager_pid = create_filter (get_pager (), -1, &outfd);
+    create_filter (get_pager (), -1, &outfd);
   
   pid = create_filter (view_command, outfd, pfd);
   if (pid > 0)
@@ -577,7 +596,7 @@ run_mailcap (mu_mailcap_entry_t entry, struct mime_context *ctx)
 	    mu_error ("waitpid: %s", mu_strerror (errno));
 	    break;
 	  }
-      if (ctx->debug_level)
+      if (mu_debug_level_p (ctx->dh, MU_DEBUG_TRACE0))
 	print_exit_status (status);
     }
   return 0;
@@ -591,21 +610,12 @@ find_entry (const char *file, struct mime_context *ctx)
   mu_stream_t stream;
   int rc = 1;
 
-  DEBUG (ctx, 2, (_("Trying %s...\n"), file));
+  mu_debug (ctx->dh, MU_DEBUG_TRACE1, (_("trying %s...\n"), file));
   status = mu_file_stream_create (&stream, file, MU_STREAM_READ);
   if (status)
     {
-      mu_error ("cannot create file stream %s: %s",
-		file, mu_strerror (status));
-      return -1;
-    }
-
-  status = mu_stream_open (stream);
-  if (status)
-    {
-      mu_stream_destroy (&stream, mu_stream_get_owner (stream));
       if (status != ENOENT)
-	mu_error ("cannot open file stream %s: %s",
+	mu_error ("cannot open input stream %s: %s",
 		  file, mu_strerror (status));
       return -1;
     }
@@ -633,7 +643,7 @@ find_entry (const char *file, struct mime_context *ctx)
 	  
 	  if (fnmatch (buffer, type, FNM_CASEFOLD) == 0)
 	    {
-	      DEBUG (ctx, 2, (_("Found in %s\n"), file));
+	      mu_debug (ctx->dh, MU_DEBUG_TRACE1, (_("found in %s\n"), file));
 	      if (run_mailcap (entry, ctx) == 0)
                 {
 		  rc = 0;
@@ -654,36 +664,51 @@ find_entry (const char *file, struct mime_context *ctx)
 int
 display_stream_mailcap (const char *ident, mu_stream_t stream, mu_header_t hdr,
 			const char *no_ask, int interactive, int dry_run,
-			int debug_level)
+			mu_debug_handle_t dh)
 {
-  char *p, *sp;
-  char *mailcap_path;
+  char *mailcap_path, *mailcap_path_tmp = NULL;
+  struct mu_wordsplit ws;
   struct mime_context ctx;
   int rc = 1;
   
   if (mime_context_fill (&ctx, ident, stream, hdr,
-			 no_ask, interactive, dry_run, debug_level))
+			 no_ask, interactive, dry_run, dh))
     return 1;
   mailcap_path = getenv ("MAILCAP");
   if (!mailcap_path)
     {
       char *home = mu_get_homedir ();
-      asprintf (&mailcap_path, "%s/.mailcap:%s", home, DEFAULT_MAILCAP);
+      mailcap_path_tmp = mu_make_file_name_suf (home, ".mailcap:",
+						DEFAULT_MAILCAP);
       free (home);
+      if (!mailcap_path_tmp)
+        return 1;
+      mailcap_path = mailcap_path_tmp;
+    }
+  
+  mu_opool_create (&expand_pool, MU_OPOOL_ENOMEMABRT);
+
+  ws.ws_delim = ":";
+  if (mu_wordsplit (mailcap_path, &ws,
+		    MU_WRDSF_DELIM|MU_WRDSF_SQUEEZE_DELIMS|
+		    MU_WRDSF_NOVAR|MU_WRDSF_NOCMD))
+    {
+      mu_error (_("cannot split line `%s': %s"), mailcap_path,
+		mu_wordsplit_strerror (&ws));
     }
   else
-    mailcap_path = strdup (mailcap_path);
-
-  obstack_init (&expand_stack);
-  
-  for (p = strtok_r (mailcap_path, ":", &sp); p; p = strtok_r (NULL, ":", &sp))
     {
-      if ((rc = find_entry (p, &ctx)) == 0)
-	break;
-    }
+      size_t i;
 
-  obstack_free (&expand_stack, NULL);
-  free (mailcap_path);
+      for (i = 0; i < ws.ws_wordc; i++)
+	{
+	  if ((rc = find_entry (ws.ws_wordv[i], &ctx)) == 0)
+	    break;
+	}
+      mu_wordsplit_free (&ws);
+    }
+  mu_opool_destroy (&expand_pool);
+  free (mailcap_path_tmp);
   mime_context_release (&ctx);
   return rc;
 }
