@@ -1,5 +1,5 @@
 /* cfg_driver.c -- Main driver for Mailutils configuration files
-   Copyright (C) 2007-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2007-2012, 2014-2018 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -161,7 +161,7 @@ dup_container (struct mu_cfg_cont **pcont)
       newcont->v.section.ident = oldcont->v.section.ident;
       newcont->v.section.label = oldcont->v.section.label;
       newcont->v.section.parser = oldcont->v.section.parser;
-      newcont->v.section.target = oldcont->v.section.target;
+      newcont->v.section.data = oldcont->v.section.data;
       newcont->v.section.offset = oldcont->v.section.offset;
       newcont->v.section.docstring = oldcont->v.section.docstring;
       newcont->v.section.children = NULL;
@@ -281,11 +281,27 @@ mu_cfg_section_add_params (struct mu_cfg_section *sect,
 		  switch (c->type)
 		    {
 		    case mu_cfg_cont_section:
-		      c->v.section.offset += param->offset;
+		      if (param->data)
+			{
+			  c->v.section.data = param->data;
+			  c->v.section.offset = param->offset;
+			}
+		      else if (c->v.section.data)
+			/* Keep data & offset */;
+		      else
+			c->v.section.offset += param->offset;
 		      break;
 
 		    case mu_cfg_cont_param:
-		      container->v.param.offset += param->offset;
+		      if (param->data)
+			{
+			  container->v.param.data = param->data;
+			  container->v.param.offset = param->offset;
+			}
+		      else if (container->v.param.data)
+			/* Keep data & offset */;
+		      else
+			container->v.param.offset += param->offset;
 		      break;
 		    }
 		  mu_cfg_section_add_container (sect, c);
@@ -298,7 +314,7 @@ mu_cfg_section_add_params (struct mu_cfg_section *sect,
 	      mu_config_clone_container (container);
 	      if (mu_refcount_value (container->refcount) > 1)
 		dup_container (&container);
-	      container->v.section.target = param->data;
+	      container->v.section.data = param->data;
 	      container->v.section.offset = param->offset;
 	    }
 	}
@@ -598,14 +614,14 @@ struct mapping_closure
 };
 
 static int
-parse_mapping (void *item, void *data)
+parse_mapping_str (void *item, void *data)
 {
   struct mapping_closure *clos = data;
-  char *str = item;
+  char const *str = item;
   size_t len;
   char *key, *val;
   int rc;
-  
+
   len = strcspn (str, "=");
   if (str[len] == 0)
     {
@@ -623,6 +639,16 @@ parse_mapping (void *item, void *data)
   return rc;
 }
 
+static int
+parse_mapping_val (void *item, void *data)
+{
+  struct mu_config_value *cval = item;
+
+  if (mu_cfg_assert_value_type (cval, MU_CFG_STRING))
+    return MU_ERR_PARSE;
+  return parse_mapping_str ((void*) cval->v.string, data);
+}
+
 int
 mu_cfg_field_map (struct mu_config_value const *val, mu_assoc_t *passoc,
 		  char **err_term)
@@ -644,12 +670,12 @@ mu_cfg_field_map (struct mu_config_value const *val, mu_assoc_t *passoc,
       mu_list_set_destroy_item (list, mu_list_free_item);
       rc = mu_string_split (val->v.string, ":", list);
       if (rc == 0)
-	rc = mu_list_foreach (list, parse_mapping, &clos);
+	rc = mu_list_foreach (list, parse_mapping_str, &clos);
       mu_list_destroy (&list);
       break;
 
     case MU_CFG_LIST:
-      rc = mu_list_foreach (val->v.list, parse_mapping, &clos);
+      rc = mu_list_foreach (val->v.list, parse_mapping_val, &clos);
       break;
 
     case MU_CFG_ARRAY:
