@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2004-2012, 2014-2017 Free Software
+   Copyright (C) 1999-2002, 2004-2012, 2014-2018 Free Software
    Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -56,7 +56,7 @@ mu_sieve_require (mu_sieve_machine_t mach, mu_sieve_slice_t list)
 
       if (rc)
 	{
-	  mu_diag_at_locus (MU_LOG_ERROR, &mach->locus, _("can't require %s"),
+	  mu_diag_at_locus_range (MU_LOG_ERROR, &mach->locus, _("can't require %s"),
 			    name);
 	  mu_i_sv_error (mach);
 	}
