@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 2005, 2007, 2010, 2011 Free
+   Copyright (C) 1999-2001, 2003, 2005, 2007, 2010-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -320,7 +320,7 @@ mbox_scan_internal (mu_mailbox_t mailbox, mbox_message_t mum,
   int newline;
   size_t n = 0;
   mu_stream_t stream;
-  size_t min_uid = 1;
+  size_t min_uid = 0;
   int zn, isfrom = 0;
   char *temp;
   
@@ -367,14 +367,12 @@ mbox_scan_internal (mu_mailbox_t mailbox, mbox_message_t mum,
 		  mum->body_end = total - n - newline;
 		  mum->body_lines = --lines - newline;
 
-		  if (mum->uid < min_uid)
+		  if (mum->uid == 0)
 		    {
-		      mum->uid = min_uid++;
+		      mum->uid = ++min_uid;
 		      /* Note that modification for when expunging.  */
 		      mum->attr_flags |= MU_ATTRIBUTE_MODIFIED;
 		    }
-		  else
-		    min_uid = mum->uid;
 
 		  if (flags & MBOX_SCAN_ONEMSG)
 		    break;
@@ -403,7 +401,7 @@ mbox_scan_internal (mu_mailbox_t mailbox, mbox_message_t mum,
 	    {
 	      char *p;
 	      unsigned long n = strtoul (buf + 6, &p, 10);
-	      if (*p == 0 || mu_isspace (*p))
+	      if ((*p == 0 || mu_isspace (*p)) && n > min_uid)
 		mum->uid = min_uid = n;
 	    }
 	  else if (mud->messages_count == 1 && IS_X_IMAPBASE (buf))
@@ -451,14 +449,12 @@ mbox_scan_internal (mu_mailbox_t mailbox, mbox_message_t mum,
       mum->body_end = total - newline;
       mum->body_lines = lines - newline;
 
-      if (mum->uid < min_uid)
+      if (mum->uid == 0)
 	{
-	  mum->uid = min_uid++;
+	  mum->uid = ++min_uid;
 	  /* Note that modification for when expunging.  */
 	  mum->attr_flags |= MU_ATTRIBUTE_MODIFIED;
 	}
-      else
-	min_uid = mum->uid;
       
       if (flags & MBOX_SCAN_NOTIFY)
 	DISPATCH_ADD_MSG (mailbox, mud);
