@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2002, 2005, 2007, 2010-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -24,41 +23,43 @@ static char *prev_name;
  * #	    the previous file
  * &	    the current mbox
  * +file    the file named in the folder directory (set folder=foo)
- * Note 1) The followig notations are left intact, since they are
- * handled by mailbox_create_default:
  * %	    system mailbox
- * %user    system mailbox of the user 
- * Note 2) Allocates memory
+ * %user    system mailbox of the user
  */
 char *
 mail_expand_name (const char *name)
 {
-  switch (name[0])
+  int status = 0;
+  char *exp = NULL;
+    
+  if (strcmp (name, "#") == 0)
     {
-    case '#':
       if (!prev_name)
-	util_error (_("No previous file"));
+	{
+	  mu_error (_("No previous file"));
+	  return NULL;
+	}
       else
-	name = xstrdup (prev_name);
-      break;
-	  
-    case '&':
+	return mu_strdup (prev_name);
+    }
+
+  if (strcmp (name, "&") == 0)
+    {
       name = getenv ("MBOX");
       if (!name)
-	util_error (_("MBOX environment variable not set"));
-      else
-	name = xstrdup (name);
-      break;
-	  
-    case '+':
-      name = util_folder_path (name);
-      break;
-
-    default:
-      name = xstrdup (name);
-      break;
+	{
+	  mu_error (_("MBOX environment variable not set"));
+	  return NULL;
+	}
+      /* else fall through */
     }
-  return (char*) name;
+
+  status = mu_mailbox_expand_name (name, &exp);
+
+  if (status)
+    mu_error (_("Failed to expand %s: %s"), name, mu_strerror (status));
+
+  return (char*) exp;
 }
 
 /*
@@ -77,33 +78,34 @@ mail_file (int argc, char **argv)
     {
       /* switch folders */
       char *pname;
-      url_t url;
-      mailbox_t newbox = NULL;
+      mu_url_t url;
+      mu_mailbox_t newbox = NULL;
       char *name = mail_expand_name (argv[1]);
       int status;
 
       if (!name)
 	return 1;
       
-      if ((status = mailbox_create_default (&newbox, name)) != 0 
-	  || (status = mailbox_open (newbox, MU_STREAM_RDWR)) != 0)
+      if ((status = mu_mailbox_create (&newbox, name)) != 0 
+	  || (status = mu_mailbox_open (newbox, MU_STREAM_RDWR)) != 0)
 	{
-	  mailbox_destroy (&newbox);
-	  util_error(_("Cannot open mailbox %s: %s"), name, mu_strerror (status));
+	  mu_mailbox_destroy (&newbox);
+	  mu_error(_("Cannot open mailbox %s: %s"), name, mu_strerror (status));
 	  free (name);
 	  return 1;
 	}
 
       free (name); /* won't need it any more */
-
-      mailbox_get_url (mbox, &url);
-      pname = strdup (url_to_string (url));
+      page_invalidate (1); /* Invalidate current page map */
+      
+      mu_mailbox_get_url (mbox, &url);
+      pname = mu_strdup (mu_url_to_string (url));
       if (mail_mbox_close ())
 	{
 	  if (pname)
 	    free (pname);
-	  mailbox_close (newbox);
-	  mailbox_destroy (&newbox);
+	  mu_mailbox_close (newbox);
+	  mu_mailbox_destroy (&newbox);
 	  return 1;
 	}
       
@@ -112,18 +114,19 @@ mail_file (int argc, char **argv)
       prev_name = pname;
       
       mbox = newbox;
-      mailbox_messages_count (mbox, &total);
-      cursor = 1;
-      if (util_getenv (NULL, "header", Mail_env_boolean, 0) == 0)
+      mu_mailbox_messages_count (mbox, &total);
+      set_cursor (1);
+      if (mailvar_is_true ("header"))
 	{
 	  util_do_command ("summary");
-	  util_do_command ("z.");
+	  util_do_command ("headers");
 	}
       return 0;
     }
   else
     {
-      util_error(_("%s takes only one argument"), argv[0]);
+      mu_error (_("%s takes only one argument"), argv[0]);
     }
   return 1;
 }
+
