@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007, 2010-2012, 2014-2016 Free Software
+   Copyright (C) 2003, 2007, 2010-2012, 2014-2018 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -74,6 +74,9 @@ check_draft_disposition (struct mh_whatnow_env *wh, int use_draft)
   struct stat st;
   int disp = DISP_REPLACE;
 
+  if (wh->nowhatnowproc)
+    return disp;
+  
   /* First check if the draft exists */
   if (stat (wh->draftfile, &st) == 0)
     {
