@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2007, 2010-2012, 2014-2017 Free
+   Copyright (C) 1999-2003, 2005-2007, 2010-2012, 2014-2018 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include <mailutils/wordsplit.h>
 #include <mailutils/io.h>
 #include <mailutils/cli.h>
+#include <mailutils/gitinfo.h>
 
 struct getopt_data
 {
@@ -91,7 +92,7 @@ process_std_options (int argc, char **argv, struct mu_parseopt *po)
     }
   if (strcmp (argv[0], "--version") == 0)
     {
-      mu_program_version (po, mu_strout);
+      mu_version_hook (po, mu_strout);
       exit (0);
     }
 }
@@ -144,13 +145,13 @@ static struct mu_option folder_option[] = {
 void
 mh_version_hook (struct mu_parseopt *po, mu_stream_t stream)
 {
-#ifdef GIT_DESCRIBE
-  mu_stream_printf (stream, "%s (%s %s) [%s]\n",
-		    mu_program_name, PACKAGE_NAME, PACKAGE_VERSION,
-		    GIT_DESCRIBE);
-#else
   mu_stream_printf (stream, "%s (%s %s)\n", mu_program_name,
 		    PACKAGE_NAME, PACKAGE_VERSION);
+#if MU_GIT_COMMIT_DISTANCE > 0  
+  mu_stream_printf (stream, "version: %s %s-%d [%s]\n",
+		    PACKAGE_NAME, PACKAGE_VERSION,
+		    MU_GIT_COMMIT_DISTANCE,
+		    MU_GIT_DESCRIBE_STRING);
 #endif
   /* TRANSLATORS: Translate "(C)" to the copyright symbol
      (C-in-a-circle), if this symbol is available in the user's
@@ -177,10 +178,39 @@ has_folder_option (struct mu_option *opt)
   return 0;
 }
 
+static void
+opt_init (struct mu_parseopt *po,
+	  struct mu_option **optv, struct mh_optinit *optinit)
+{
+  if (!optinit)
+    return;
+  for (; optinit->opt; optinit++)
+    {
+      size_t i;
+      for (i = 0; optv[i]; i++)
+	{
+	  struct mu_option *opt;
+	  for (opt = optv[i]; !MU_OPTION_IS_END (opt); opt++)
+	    {
+	      if (strcmp (opt->opt_long, optinit->opt) == 0)
+		{
+		  char const *val = mh_global_profile_get (optinit->var, NULL);
+		  if (val)
+		    {
+		      (opt->opt_set ?
+		       opt->opt_set : mu_option_set_value) (po, opt, val);
+		    }
+		  break;
+		}
+	    }
+	}
+    }
+}
+
 void
-mh_getopt (int *pargc, char ***pargv, struct mu_option *options,
-	   int mhflags,
-	   char *argdoc, char *progdoc, char *extradoc)
+mh_getopt_ext (int *pargc, char ***pargv, struct mu_option *options,
+	       int mhflags, struct mh_optinit *optinit,
+	       char *argdoc, char *progdoc, char *extradoc)
 {
   int argc = *pargc;
   char **argv = *pargv;
@@ -254,6 +284,7 @@ mh_getopt (int *pargc, char ***pargv, struct mu_option *options,
   if (options)
     optv[i++] = options;
   optv[i] = NULL;
+  opt_init (&po, optv, optinit);
   
   if (mu_parseopt (&po, argc, argv, optv, flags))
     exit (po.po_exit_error);
@@ -282,6 +313,14 @@ mh_getopt (int *pargc, char ***pargv, struct mu_option *options,
   mh_init2 ();
 }
 
+void
+mh_getopt (int *pargc, char ***pargv, struct mu_option *options,
+	   int mhflags, char *argdoc, char *progdoc, char *extradoc)
+{
+  mh_getopt_ext (pargc, pargv, options, mhflags, NULL, argdoc, progdoc,
+		 extradoc);
+}
+
 void
 mh_opt_notimpl (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
@@ -324,3 +363,21 @@ mh_opt_read_formfile (struct mu_parseopt *po, struct mu_option *opt,
 {
   mh_read_formfile (arg, opt->opt_ptr);
 }
+
+void
+mh_opt_parse_formfile (struct mu_parseopt *po, struct mu_option *opt,
+		       char const *arg)
+{
+  mh_format_destroy ((mh_format_t*) opt->opt_ptr);
+  if (mh_format_file_parse (opt->opt_ptr, arg, MH_FMT_PARSE_DEFAULT))
+    exit (1);
+}
+
+void
+mh_opt_parse_format (struct mu_parseopt *po, struct mu_option *opt,
+		     char const *arg)
+{
+  mh_format_destroy ((mh_format_t*) opt->opt_ptr);
+  if (mh_format_string_parse (opt->opt_ptr, arg, NULL, MH_FMT_PARSE_DEFAULT))
+    exit (1);
+}
