@@ -1,10 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005,
-   2006 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2005-2007, 2010-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* This file implements an MH draftfile stream: a read-only stream used
    to transparently pass MH draftfiles to mailers. The only difference
@@ -32,6 +30,7 @@ mh_stream_to_message (mu_stream_t instream)
   mu_message_t msg;
 
   rc = mu_stream_to_message (instream, &msg);
+  mu_stream_unref (instream);
   if (rc)
     {
       mu_error (_("cannot open draft message stream: %s"),
