@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2007, 2010, 2011 Free Software Foundation,
-   Inc.
+   Copyright (C) 2003, 2005, 2007, 2010-2012, 2014-2018 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -56,6 +56,6 @@ struct node
   } v;
 };
 
-void pick_add_token (mu_list_t *list, int tok, char *val);
+void pick_add_token (mu_list_t *list, int tok, char const *val);
 int pick_parse (mu_list_t toklist);
 int pick_eval (mu_message_t msg);
