@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2007-2012, 2014-2016 Free Software
+   Copyright (C) 2002, 2005, 2007-2012, 2014-2018 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -31,30 +31,12 @@ rmm (size_t num, mu_message_t msg, void *data)
   return 0;
 }
 
-struct seq_closure
-{
-  mu_msgset_t rmset;
-  int rmflag;
-};
-
-static int
-rmseq (const char *name, const char *value, void *data)
-{
-  struct seq_closure *s = data;
-  mu_mailbox_t mbox;
-
-  mu_msgset_sget_mailbox (s->rmset, &mbox);
-  mh_seq_delete (mbox, name, s->rmset, s->rmflag);
-  return 0;
-}
-
 int
 main (int argc, char **argv)
 {
   mu_mailbox_t mbox;
   mu_msgset_t msgset;
   int status;
-  struct seq_closure clos;
   
   mh_getopt (&argc, &argv, NULL, MH_GETOPT_DEFAULT_FOLDER,
 	     args_doc, prog_doc, NULL);
@@ -64,17 +46,13 @@ main (int argc, char **argv)
   mh_msgset_parse (&msgset, mbox, argc, argv, "cur");
 
   status = mu_msgset_foreach_message (msgset, rmm, NULL);
-  
-  clos.rmset = msgset;
-  clos.rmflag = 0;
-  mh_global_sequences_iterate (mbox, rmseq, &clos);
-  clos.rmflag = SEQ_PRIVATE;
-  mh_private_sequences_iterate (mbox, rmseq, &clos);
 
+  mh_sequences_elim (msgset);
+  
   mu_mailbox_expunge (mbox);
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
   mh_global_save_state ();
-  return status;
+  return !!status;
 }
 
