@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2003, 2005, 2007, 2009-2012, 2014-2018 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -49,9 +49,7 @@ pop3d_begin_session ()
       return ERR_MBOX_LOCK;
     }
   
-  username = strdup (auth_data->name);
-  if (username == NULL)
-    pop3d_abquit (ERR_NO_MEM);
+  username = mu_strdup (auth_data->name);
   state = TRANSACTION;
 
   pop3d_outf ("+OK opened mailbox for %s\n", username);
@@ -74,7 +72,7 @@ pop3d_begin_session ()
 }
 
 int
-pop3d_user (char *arg)
+pop3d_user (char *arg, struct pop3d_session *sess)
 {
   char *buf, *pass, *cmd;
   char buffer[512];
@@ -109,29 +107,43 @@ pop3d_user (char *arg)
 	}
 #endif
 
-      auth_data = mu_get_auth_by_name (arg);
-
-      if (auth_data == NULL)
+      rc = mu_get_auth (&auth_data, mu_auth_key_name, arg);
+      switch (rc)
 	{
+	case 0:
+	  break;
+	case MU_ERR_AUTH_FAILURE:
+	case MU_ERR_NOENT:
 	  mu_diag_output (MU_DIAG_INFO, _("user `%s' nonexistent"), arg);
 	  return ERR_BAD_LOGIN;
+	default:
+	  mu_error (_("error getting identity info for user `%s': %s"),
+		    arg, mu_strerror (rc));
+	  return ERR_SYS_LOGIN;
 	}
 
       rc = mu_authenticate (auth_data, pass);
       openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
 
-      if (rc)
+      switch (rc)
 	{
+	case 0:
+	  break;
+	case MU_ERR_AUTH_FAILURE:
 	  mu_diag_output (MU_DIAG_INFO,
 			  _("user `%s': authentication failed"), arg);
 	  mu_auth_data_destroy (&auth_data);
 	  return ERR_BAD_LOGIN;
+	default:
+	  mu_error (_("error authenticating user `%s': %s"),
+		    arg, mu_strerror (rc));
+	  return ERR_SYS_LOGIN;
 	}
     }
   else if (mu_c_strcasecmp (cmd, "QUIT") == 0)
     {
       mu_diag_output (MU_DIAG_INFO, _("possible probe of account `%s'"), arg);
-      return pop3d_quit (pass);
+      return pop3d_quit (pass, sess);
     }
   else
     {
